"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
class LeanHoverProvider {
    constructor(server) {
        this.server = server;
    }
    provideHover(document, position) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.server.info(document.fileName, position.line + 1, position.character);
            if (response.record) {
                const contents = [];
                const name = response.record['full-id'] || response.record.text;
                if (name) {
                    if (response.record.tactic_params) {
                        contents.push(new vscode_1.MarkdownString()
                            .appendText(name + ' ' + response.record.tactic_params.join(' ')));
                    }
                    else {
                        contents.push(new vscode_1.MarkdownString()
                            .appendCodeblock(name + ' : ' + response.record.type, 'lean'));
                    }
                }
                if (response.record.doc) {
                    contents.push(new vscode_1.MarkdownString()
                        .appendMarkdown(response.record.doc));
                }
                if (response.record.state && !contents) {
                    contents.push(new vscode_1.MarkdownString()
                        .appendCodeblock(response.record.state, 'lean'));
                }
                const pos = new vscode_1.Position(position.line - 1, position.character);
                return new vscode_1.Hover(contents, new vscode_1.Range(pos, pos));
            }
        });
    }
}
exports.LeanHoverProvider = LeanHoverProvider;
//# sourceMappingURL=hover.js.map